"""Script to store static data."""

OP = [
    "FETCHING",
    "PARSING",
    "RETRY FETCHING",
    "RETRY PARSING",
    "PRINTING",
    "FOUND",
    "NOT FOUND",
    "CACHED",
    "CANCELLED",
    "DELETED",
]

DICTS = ["CAMBRIDGE", "WEBSTER"]
